<?php

/*
    Plugin Name: MDHQ Gravity Form Relay
    Plugin URI: https://fxmedsupport.com
    Description: Gravity Form Extension For pushing to MDHQ
    Version: 1.3
    Author: Muhammad Yaseen
*/

defined( 'ABSPATH' ) || die('No direct access');

if(!class_exists('GravityFormExtension')) {

    class GravityFormExtension
    {
        public function __construct()
        {
            date_default_timezone_set(get_option('timezone_string'));
            add_action( 'gform_after_submission', array($this, 'after_submission'), 1000, 2 );
            add_action( 'gform_entries_first_column_actions', array($this, 'add_button_first_column'), 1000, 5 );
            add_action( 'init', array($this, 'handle_save_mdhq'));
            add_action( 'rest_api_init', function () {
                register_rest_route( 'gfe/v1', '/forms', array(
                    'methods' => 'GET',
                    'callback' => array($this, 'get_all_forms'),
                    'permission_callback' => '__return_true',
                ) );
            } );
            add_action( 'rest_api_init', function () {
                register_rest_route( 'gfe/v1', '/forms/fields', array(
                    'methods' => 'GET',
                    'callback' => array($this, 'get_form_with_fields'),
                    'permission_callback' => '__return_true',
                ) );
            } );
            add_action( 'rest_api_init', function () {
                register_rest_route( 'gfe/v1', '/entries', array(
                    'methods' => 'GET',
                    'callback' => array($this, 'handle_fxmed_request'),
                    'permission_callback' => '__return_true',
                ) );
            } );
        }

        public function get_all_forms(WP_REST_Request $request)
        {
            $token = $request->get_header('token');
            if ($token !== "8b2f95580ce664215bc53cb250dbe34a") {
                return ['success' => 0, 'error' => "Authentication Failed"];
            }

            $forms = GFAPI::get_forms();
            $results = [];
            foreach ($forms as $form) {
                $results[] = ['id' => $form['id'], 'title' => $form['title']];
            }
            return ['success' => 1, 'data' => json_encode($results)];
        }
        
        public function get_form_with_fields(WP_REST_Request $request)
        {
            $token = $request->get_header('token');
            if ($token !== "8b2f95580ce664215bc53cb250dbe34a") {
                return ['success' => 0, 'error' => "Authentication Failed"];
            }
            
            if (empty($request['form_id'])) {
                return ['success' => 0, 'error' => "Missing Required fields"];
            }

            $form = GFAPI::get_form( $request['form_id'] );
            return ['success' => 1, 'data' => $form];
        }

        public function add_button_first_column($form_id, $field_id, $value, $entry, $query_string)
        {
            $id = $entry['id'];
            echo " | <a href='?$query_string&push_mdhq_id=$form_id&entry_id=$id'>Push To MDHQ</a>";
        }

        public function handle_fxmed_request(WP_REST_Request $request)
        {
            $token = $request->get_header('token');
            if (empty($request['email']) || empty($token) || empty($request['form_id'])) {
                return ['success' => 0, 'error' => "Missing Required fields"];
            }

            if (md5($request['email']) !== $token) {
                return ['success' => 0, 'error' => "Authentication Failed"];
            }

            $form = GFAPI::get_form( $_GET['form_id'] );

            if (empty($form)) {
                return ['success' => 0, 'error' => "Invalid form id provided"];
            }

            $emailField = false;
            foreach ($form['fields'] as $field) {
                if (strtolower($field['type']) === 'email' || strtolower($field['label']) === 'email') {
                    $emailField = $field['id'];
                    break;
                }
            }

            if ($emailField === false) {
                return ['success' => 0, 'error' => "Form does not have email field"];
            }

            $search_criteria['field_filters'][] = array('key' => $emailField, 'value' => $_GET['email']);
            $entries = GFAPI::get_entries( $form['id'], $search_criteria);
            foreach ($entries as $entry) {
                $data[] = $this->format_data($form, $entry);
            }

            return ['success' => 1, 'data' => $data];
        }

        private function format_data($form, $entry)
        {
            $gf_html = GFCommon::get_submitted_fields( $form, $entry, true, true, 'html', false, 'all_fields_display_empty' );
            $extendData = array();
            $data = array();
            foreach ($form['fields'] as $field) {
                if($field->type === 'name') {
                    $value = [];
                    foreach ($field->inputs as $input) {
                        if(!empty($entry[$input['id']])) {
                            $value[] = $entry[$input['id']];   
                        }
                    }
                    $value = implode(' ', $value);
                } else {
                    $value = $entry[$field->id];
                }
                $data[$field->label] = $value;
                $extendData[$field->id] = ['answer' => $value, 'label' => $field->label];
            }
            $params = array('html' => $gf_html, 'json' => json_encode($data), 'form_name' => $form['title'], 'extended_data' => $extendData);
            return $params;
        }

        public function handle_save_mdhq()
        {
            if (!empty($_GET['push_mdhq_id'])) {
                $form_id = $_GET['push_mdhq_id'];
                $entry_id = $_GET['entry_id'];
                $form = GFAPI::get_form( $form_id );
                $entry = GFAPI::get_entry( $entry_id );
                $result = $this->after_submission($entry, $form, true);
                $result = json_decode($result, true);
                if (!session_id()) {
                    session_start();
                }
                if ((int) $result['success'] === 1) {
                    $message = 'Data is successfully pushed to MDHQ.';
                    $_SESSION['success-mdhq'] = $message;
                } else {
                    $message = empty($result) ? "Something went wrong. Please try later" : $result['error'];
                    $_SESSION['error-mdhq'] = $message;
                }

                if (!empty($_SERVER['HTTP_REFERER'])) {
                    $url = $_SERVER['HTTP_REFERER'];
                } else {
                    $url = rtrim(get_site_url(), "/") . "/wp-admin/admin.php?page=gf_entries";
                }

                header('Location: '.$url);
                exit();
            }

            if (!empty($_GET['page']) && $_GET['page'] === 'gf_entries') {
                if (!session_id()) {
                    session_start();
                }
                if (!empty($_SESSION['success-mdhq'])) {
                    echo '<div class="updated"><p>' . $_SESSION['success-mdhq'] . '</p></div>';
                    unset($_SESSION['success-mdhq']);
                }
                if (!empty($_SESSION['error-mdhq'])) {
                    echo '<div class="error"><p>' . $_SESSION['error-mdhq'] . '</p></div>';
                    unset($_SESSION['error-mdhq']);
                }
            }
        }

        public function after_submission( $entry, $form, $manual_entry = false ) {
            $gf_html = GFCommon::get_submitted_fields( $form, $entry, true, true, 'html', false, 'all_fields_display_empty' );
            $site = get_site_url();
            $send = false;
            $data = array();
            $extendData = array();
            foreach ($form['fields'] as $field) {
                if (strtolower($field->label) === 'email') {
                    $send = $entry[$field->id];
                }
                if($field->type === 'name') {
                    $value = [];
                    foreach ($field->inputs as $input) {
                        if(!empty($entry[$input['id']])) {
                            $value[] = $entry[$input['id']];   
                        }
                    }
                    $value = implode(' ', $value);
                } else {
                    $value = $entry[$field->id];
                }
                $data[$field->label] = $value;
                $extendData[$field->id] = ['answer' => $value, 'label' => $field->label];
            }

            if (!empty($send)) {
                $params = array('email' => $send, 'site_url' => $site, 'html' => $gf_html, 'json' => json_encode($data), 'form_name' => $form['title'], 'manual_entry' => $manual_entry, 'form_id' => $form['id'], 'extended_data' => $extendData);
                return $this->send_to_fxmedsupport($params);
            }

            return json_encode(array("success" => 0, "error" => "Form must have email field"));
        }

        public function send_to_fxmedsupport($params)
        {
            $url = 'https://app.fxmedsupport.com/webhook/gravity-form';
            return $this->post_curl($url, $params);
        }

        public function post_curl($url, $params)
        {
            $ch = curl_init();

            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt( $ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));

            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

            $server_output = curl_exec($ch);

            curl_close ($ch);

            return $server_output;
        }
    }
}

new GravityFormExtension();

