<?php

if(!class_exists('MDHQCRelayFileLogger')) {
    class MDHQCRelayFileLogger
    {
        const logFileName = 'mdhq_log.log';
        const notFoundFileName = 'mdhq_not_found_log.log';
        const foundFileName = 'mdhq_found_log.log';
        const productLog = 'product_log.log';
        const openMode = 'a+';

        private $path = '';

        private $logFile = '';
        private $foundFile = '';
        private $notFoundFile = '';
        private $productFile = '';

        public function __construct($path = 'log')
        {
            $this->path = __DIR__ . '/' . $path . '/';
        }
        public function __destruct()
        {
            if (is_resource($this->logFile))
                fclose($this->logFile);
            if (is_resource($this->notFoundFile))
                fclose($this->notFoundFile);
            if (is_resource($this->foundFile))
                fclose($this->foundFile);
            if (is_resource($this->productFile))
                fclose($this->productFile);
        }
        public function log($data, $type = 'log', $level = 'INFO')
        {
            if(!is_string($data))
                return false;

            $msg = '[' . strtoupper($level) . '] ' . date('Y-m-d H:i:s') . ' - ';
            $msg .= $data;
            $msg .= "\n";

            $methodName = '_' . $type;
            call_user_func([$this, $methodName], $msg);
        }
        public function getLog($type = 'log') {
            $fileName = $this->_logFileSelection($type);
            if(!file_exists($this->path . $fileName))
                return array();
            $handle = fopen($this->path . $fileName, "r");
            if(!$handle) {
                return array();
            }
            while(!feof($handle)) {
                yield trim(fgets($handle));
            }
            fclose($handle);
        }
        public function resetLog($type = 'log') {
            $fileName = $this->_logFileSelection($type);
            $f = @fopen($this->path . $fileName, "r+");
            if ($f !== false) {
                ftruncate($f, 0);
                fclose($f);
            }
        }
        public function download($type = 'log') {
            $name = $this->_logFileSelection($type);
            $file = $this->path . $name;

            $contents = file_get_contents($file);

            header('Content-type: text/plain');
            header('Content-Disposition: attachment; filename="' . $name .'"');
            return $contents;
        }
        private function _logFileSelection($type = 'log') {
            $fileName = '';
            switch($type) {
                case 'log': $fileName = self::logFileName;
                    break;
                case 'logFound': $fileName = self::foundFileName;
                    break;
                case 'logNotFound': $fileName = self::notFoundFileName;
                    break;
                case 'product': $fileName = self::productLog;
                    break;
            }
            return $fileName;
        }
        private function _log($message = '')
        {
            if (!$this->logFile) {
                $this->logFile = fopen($this->path . self::logFileName, self::openMode);
            }
            fwrite($this->logFile, $message);
        }

        private function _logFound($message = '')
        {
            if (!$this->foundFile) {
                $this->foundFile = fopen($this->path . self::foundFileName, self::openMode);
            }
            fwrite($this->foundFile, $message);
        }

        private function _logNotFound($message = '')
        {
            if (!$this->notFoundFile) {
                $this->notFoundFile = fopen($this->path . self::notFoundFileName, self::openMode);
            }
            fwrite($this->notFoundFile, $message);
        }

        private function _product($message = '')
        {
            if (!$this->productFile) {
                $this->productFile = fopen($this->path . self::productLog, self::openMode);
            }
            fwrite($this->productFile, $message);
        }
    }
}
