<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}
if(!class_exists('MDHQCRelayProductSync')) {

    if (!class_exists('MDHQCRelayFileLogger')) {
        include __DIR__ . '/MDHQCRelayFileLogger.php';
    }

    class MDHQCRelayProductSync extends MDHQCRelayFileLogger
    {
        private $db;

        public function __construct() {
            parent::__construct();
            add_action('trashed_post', array($this, 'save') , 10, 1);
            add_action('untrashed_post', array($this, 'save') , 10, 1);
            add_action('woocommerce_update_product', array($this, 'save') , 10, 1);
            global $wpdb;
            $this->db = $wpdb;
        }

        public function save($post_id = null) {
            if(is_null($post_id)) {
                return false;
            }

            $product  = $this->db->get_row(
                "SELECT `a`.`ID`, `a`.`post_title`, `a`.`post_status`, `a`.`post_name`, `b`.`meta_value` AS `sku`, `c`.`meta_value` AS `stock`
                        FROM `{$this->db->posts}` AS `a` 
                        LEFT JOIN `{$this->db->postmeta}` AS `b` ON `a`.`ID` = `b`.`post_id` AND `b`.`meta_key` LIKE '_sku' 
                        LEFT JOIN `{$this->db->postmeta}` AS `c` ON `a`.`ID` = `c`.`post_id` AND `c`.`meta_key` LIKE '_stock_status' 
                        WHERE `a`.`post_type` LIKE 'product' AND
                              `a`.`ID` = '{$post_id}'"
            );
            
            $taxonomy    = 'product_brand';
            $brand_names = wp_get_post_terms( $post_id, $taxonomy, array( 'fields' => 'names' ) );
            
            // Get the brand name
            $brand_name = reset( $brand_names );

            if(!empty($product)) {
                if (empty($product->post_name) || empty($product->sku)) {
                    return false;
                }

                $sku 	= $product->sku;
                $stock 	= $product->stock;
                $brand  = empty($brand_name) ? 'NO BRAND' : $brand_name;
                $name = $product->post_title . ' - ' . $sku . ' - ' . $brand;
                $slug = str_ireplace('__trashed','',$product->post_name);
                $nickname = str_ireplace('-',' ', $slug) . ', ' . $sku . ', ' . $brand;

                $inactive = true;
                if($product->post_status === 'publish' && $stock === 'instock') {
                    $inactive = false;
                }
                $data_string = array(
                    'name' 			=> $name, // update the title of the product
                    'nicknames' 	=> $nickname,
                    'vendor_code' 	=> $sku,
                    'description' 	=> '',
                    'class' 		=> '',
                    'external_ref_id' => $product->ID,
                    'inactive' 		=> $inactive,
                    'site' => get_site_url(),
                );

                $updated = $this->postCurl('https://nutrimentrx.com/?receive_product_mdhqc=1', json_encode($data_string));
                $updated = json_decode($updated);
                if(!empty($updated) && !isset($updated->error)) {
                    $this->log($sku . '-' . $name . ' product pushed' , 'product');
                } else {
                    $this->log($sku . '-' . $name . ' product not pushed' , 'product');
                }
                return 1;
            }

            return 0;
        }

        public function postCurl($url, $params)
        {
            $ch = curl_init();

            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt( $ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
            curl_setopt($ch, CURLOPT_POSTFIELDS, $params);

            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

            $server_output = curl_exec($ch);

            curl_close ($ch);

            return $server_output;
        }
    }
}
