<?php

defined( 'ABSPATH' ) || die('No direct access');

define('SYNC_RELAY_SERVER_DIR',        __DIR__);

if(!class_exists('MDHQCRelaySyncNrx')) {
    class MDHQCRelaySyncNrx
    {

        const RELAY_SERVER_ENDPOINTS = '/api/sync-product';
        const SKIP_META = [
            '_thumbnail_id',
            '_edit_lock',
            '_edit_last',
            '_related_product_v2',
            '_related_product_type',
            'CM_update_products_batch'
        ];

        const VENDOR_TEXONOMY = "wcpv_product_vendors";
        const CATEGORY_TEXONOMY = "product_cat";
        const BRAND_TEXONOMY = "product_brand";
        const TAG_TEXONOMY = "product_tag";
        const SHIPPING_TEXONOMY = "product_shipping_class";

        public function __construct()
        {
            date_default_timezone_set(get_option('timezone_string'));
            if (!is_admin()) {
                add_action('init', array($this, 'handle_request'));
            }
        }

        /**
         * Handle request coming from NutrimentRx and process it and create/update woocommerce product
         */
        public function handle_request()
        {
            if (strpos($_SERVER['REQUEST_URI'], self::RELAY_SERVER_ENDPOINTS) === false || $_SERVER['REQUEST_METHOD'] !== 'POST')
                return;

            $inputJSON = file_get_contents('php://input');
            file_put_contents(SYNC_RELAY_SERVER_DIR . '/log/log.log', $inputJSON . "\n", FILE_APPEND);
            $data = json_decode($inputJSON, true);
            $isNew = false;
            if (!empty($data)) {
                global $wpdb;
                $isNew = false;
                if (empty($data['meta']['_sku'])) {
                    return;
                }

                $post = $this->get_product_by_sku($data['meta']['_sku'][0]);

                if(empty($post)) {
                    $prod = $data['product'];
                    unset($prod['ID']);
                    unset($prod['post_author']);
                    unset($prod['comment_count']);
                    unset($prod['post_author']);
                    unset($prod['guid']);
                    unset($prod['post_parent']);
                    $postId = wp_insert_post($prod, true);
                    /** @var WP_Error $postId */
                    if (!$postId instanceof WP_Error) {
                        $post = get_post($postId);
                        file_put_contents(SYNC_RELAY_SERVER_DIR . '/log/log.log', "\nCreated ... " . $post->ID, FILE_APPEND);
                        $isNew = true;
                    } else {
                        file_put_contents(SYNC_RELAY_SERVER_DIR . '/log/log.log', "\nresponse-error ..." . $postId->get_error_message(), FILE_APPEND);
                        die;
                    }
                } else {
                    file_put_contents(SYNC_RELAY_SERVER_DIR . '/log/log.log', "\nUpdating ... " . $post->ID, FILE_APPEND);
                    $prod = $data['product'];
                    $newData = (array) $post;
                    $newData['post_title'] = $prod['post_title'];
                    $newData['post_content'] = $prod['post_content'];
                    $newData['post_excerpt'] = $prod['post_excerpt'];
                    $newData['post_status'] = $prod['post_status'];
                    $newData['comment_status'] = $prod['comment_status'];
                    $newData['ping_status'] = $prod['ping_status'];
                    $newData['post_name'] = $prod['post_name'];
                    wp_update_post($newData);
                }

                if (wc_get_product_id_by_sku($data['meta']['_sku'][0]) === 0) {
                    $wpdb->insert(
                        $wpdb->wc_product_meta_lookup,
                        ['product_id' => $post->ID, 'sku' => $data['meta']['_sku'][0],
                            'min_price' => $data['meta']['_price'][0], 'max_price' => $data['meta']['_price'][0],
                            'stock_status' => $data['meta']['_stock_status'][0]]
                    );
                } else {
                    $wooProduct = wc_get_product($post->ID);
                    if ($wooProduct instanceof \WC_Product) {
                        $wooProduct->set_regular_price($data['meta']['_price'][0]);
                        $wooProduct->set_stock_status($data['meta']['_stock_status'][0]);
                        $wooProduct->save();
                    }
                }

                foreach ($data['meta'] as $key => $value) {
                    if (!in_array($key, self::SKIP_META)) {
                        update_post_meta($post->ID, $key, $value[0]);
                    }
                }

                $vendorData = $data['vendor'];
                if (!empty($vendorData['data']['slug'])) {
                    $existVendor = get_term_by('slug', $vendorData['data']['slug'], self::VENDOR_TEXONOMY);
                    if ($existVendor) {
                        $vendorId = $existVendor->term_id;
                    } else {
                        file_put_contents(SYNC_RELAY_SERVER_DIR . '/log/log.log', "\nCreated" . $post->ID, FILE_APPEND);
                        $newVendor = wp_insert_term($vendorData['data']['name'], self::VENDOR_TEXONOMY, [
                            'description' => $vendorData['data']['description'],
                            'slug' => $vendorData['data']['slug'],
                            'parent' => 0
                        ]);
                        $vendorId = $newVendor['term_id'];
                    }
                    wp_set_object_terms($post->ID, $vendorId, self::VENDOR_TEXONOMY);
                    foreach ($vendorData['meta'] as $key=> $value) {
                        if ($key === "vendor_data") {
                            update_term_meta($vendorId, $key, unserialize($value[0]));
                        } else {
                            update_term_meta($vendorId, $key, $value[0]);
                        }
                    }
                }

                $this->handleTermsToAttach($post->ID, $data['cats'], self::CATEGORY_TEXONOMY);
                $this->handleTermsToAttach($post->ID, $data['brand'], self::BRAND_TEXONOMY);
                $this->handleTermsToAttach($post->ID, $data['tags'], self::TAG_TEXONOMY);
                $this->handleTermsToAttach($post->ID, $data['shipping_class'], self::SHIPPING_TEXONOMY);
                if (!empty($data['image'])) {
                    $this->setThumbnail($post->ID, $data['image']);
                }
            }
            $isNew = $isNew ? "1" : "0";
            $return = ['is_new' => $isNew];
            $return = json_encode($return);
            echo $return;
            die;
        }

        /**
         * @param $productId
         * @param $tags
         * @param $texonomy
         */
        private function handleTermsToAttach($productId, $tags, $texonomy)
        {
            $brandsToAttach = [];
            if (!empty($tags)) {
                foreach ($tags as $tag) {
                    $existTag = get_term_by('slug', $tag['slug'], $texonomy);
                    if ($existTag) {
                        $brandsToAttach[] = $existTag->term_id;
                    } else {
                        $parentId = 0;
                        if (!empty($tag['parent'])) {
                            $parent = $tag['parent'];
                            $existPar = get_term_by('slug', $parent['slug'], $texonomy);
                            if ($existPar) {
                                $parentId = $existPar->term_id;
                            } else {
                                $return = wp_insert_term($parent['name'], $texonomy, [
                                    'description' => $parent['description'],
                                    'slug' => $parent['slug'],
                                    'parent' => 0
                                ]);
                                $parentId = $return['term_id'];
                            }
                        }
                        $return = wp_insert_term($tag['name'], $texonomy, [
                            'description' => $tag['description'],
                            'slug' => $tag['slug'],
                            'parent' => $parentId
                        ]);
                        $brandsToAttach[] = $return['term_id'];
                    }
                }
                if (!empty($brandsToAttach)) {
                    wp_set_object_terms($productId, $brandsToAttach, $texonomy);
                }
            }
        }

        /**
         * @param $postId
         * @param $imageUrl
         */
        private function setThumbnail($postId, $imageUrl)
        {
            $imageName       = basename($imageUrl);
            $upload_dir       = wp_upload_dir(); // Set upload folder
            $imageData       = file_get_contents($imageUrl); // Get image data
            $uniqueFileName = wp_unique_filename( $upload_dir['path'], $imageName ); // Generate unique name
            $filename         = basename( $uniqueFileName ); // Create image file name

            // Check folder permission and define file location
            if( wp_mkdir_p( $upload_dir['path'] ) ) {
                $file = $upload_dir['path'] . '/' . $filename;
            } else {
                $file = $upload_dir['basedir'] . '/' . $filename;
            }

            // Create the image  file on the server
            file_put_contents( $file, $imageData );

            // Check image file type
            $wpFiletype = wp_check_filetype( $filename, null );

            // Set attachment data
            $attachment = array(
                'post_mime_type' => $wpFiletype['type'],
                'post_title'     => sanitize_file_name( $filename ),
                'post_content'   => '',
                'post_status'    => 'inherit'
            );

            // Create the attachment
            $attachId = wp_insert_attachment( $attachment, $file, $postId );

            // Include image.php
            require_once(ABSPATH . 'wp-admin/includes/image.php');

            // Define attachment metadata
            $attachData = wp_generate_attachment_metadata( $attachId, $file );

            // Assign metadata to attachment
            wp_update_attachment_metadata( $attachId, $attachData );

            // And finally assign featured image to post
            set_post_thumbnail( $postId, $attachId );
        }

        /**
         * @param $post_title
         * @param string $output
         * @return array|WP_Post|null
         */
        private function get_post_by_title($post_title, $output = OBJECT) {
            global $wpdb;
            $post = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_title = %s AND post_type='post'", $post_title ));
            if ( $post )
                return get_post($post, $output);

            return null;
        }

        /**
         * @param $sku
         * @return array|WP_Post|null
         */
        private function get_product_by_sku( $sku ) {

            global $wpdb;

            $product_id = $wpdb->get_var( $wpdb->prepare( "SELECT post_id FROM $wpdb->postmeta WHERE meta_key='_sku' AND meta_value='%s' LIMIT 1", $sku ) );

            if ( $product_id ) return get_post($product_id);

            return null;
        }
    }
}