<?php

add_action( 'init', 'mdhqd_init', 11 );
add_action( 'mdhqd_process', 'mdhqd_process' );
register_activation_hook(__FILE__, 'mdhqd_activation');

if ( ! function_exists( 'mdhqd_init' ) ) {
    function mdhqd_init(){
        if (
            ( isset ( $_GET['curl_mdhq_wc_add_to_cart'] ) )
            || ( isset ( $_GET['curl_mdhq_wc_remove_from_cart'] ) )
        ) {
            update_user_meta(
                get_current_user_id(),
                'mdhqd_last_update',
                date( 'Y-m-d H:i:s' )
            );
        }

        if ( isset( $_GET['mdhqd_process'] ) ) {
            do_action( 'mdhqd_process' );
        }

        if ( isset( $_GET['mdhqd_send_now'] ) ) {
            header("Access-Control-Allow-Origin: *");
            add_filter( 'wp_mail_content_type', 'mdhqd_set_html_mail_content_type' );

            $hours = get_option( 'mdhqd_email_delay_hours', 0 );

            $users = get_users( array(
                'include' => array( $_GET['user_id'] ),
            ) );

            $cart_url = get_permalink( wc_get_page_id( 'cart' ) );
            $site_name = get_bloginfo( 'name' );
            $admin_email = get_option('admin_email');

            foreach ( $users as $user ) {
                $first_name = get_user_meta( $user->data->ID, 'first_name', true );
                $last_name = get_user_meta( $user->data->ID, 'last_name', true );
                $name = "{$first_name} {$last_name}";
                $nrlogin = get_user_meta( $user->data->ID, 'nr_link', true );
                $url_text = get_option( 'mdhqd_email_url_text', '' );
                $url = "<a href=\"{$cart_url}?nrlogin={$nrlogin}\">{$url_text}</a>";
                $memberships = get_posts( array(
                    'post_type' => 'wc_user_membership',
                    'post_status' => 'wcm-active',
                    'author' => $user->data->ID,
                ) );
                $user_membership = array();
                foreach ( $memberships as $membership ) {
                    $membership_info = get_post( $membership->post_parent );
                    $user_membership[] = $membership_info->post_title;
                }
                $user_membership = implode( ", ", $user_membership );
                $user_cart = get_user_meta( $user->data->ID, '_woocommerce_persistent_cart_1', true );

                if (empty($user_cart['cart'])) {
                    return;
                }

                $cart = "<table><tr><th style=\"padding:5px;background:#6691BF;color:#FFF\">Product</th><th style=\"padding:5px;background:#6691BF;color:#FFF\">Quantity</th>";
                foreach ( $user_cart['cart'] as $item_key => $item ) {
                    $background = '#C6DBF1';
                    $product = get_post( $item['product_id'] );
                    $cart .= "<tr><td style=\"background:{$background};padding:5px;\">{$product->post_title}</td><td style=\"background:{$background};padding:5px;text-align:center;\">{$item['quantity']}</td>";
                }
                $cart .= "</table>";

                $subject = get_option( 'mdhqd_email_subject', '' );
                $subject = str_replace( '[name]', $name, $subject );
                $subject = str_replace( '[1click]', $url, $subject );
                $subject = str_replace( '[membership]', $user_membership, $subject );

                $message = get_option( 'mdhqd_email_message', '' );
                $message = str_replace( '[name]', $name, $message );
                $message = str_replace( '[1click]', $url, $message );
                $message = str_replace( '[membership]', $user_membership, $message );
                $message = str_replace( '[cart]', $cart, $message );

                wp_mail(
                    $user->data->user_email,
                    $subject,
                    $message,
                    'From: '. $site_name .', supplement fulfillment center <'. $admin_email .'>'
                );

                delete_user_meta( $user->data->ID, 'mdhqd_last_update' );
            }

            remove_filter( 'wp_mail_content_type', 'mdhqd_set_html_mail_content_type' );
        }
    }
}

if ( ! function_exists( 'mdhqd_set_html_mail_content_type' ) ) {
    function mdhqd_set_html_mail_content_type() {
        return 'text/html';
    }
}

if ( ! function_exists( 'mdhqd_process' ) ) {
    function mdhqd_process() {
        if ( ! function_exists( 'get_field' ) ) {
            return;
        }

        add_filter( 'wp_mail_content_type', 'mdhqd_set_html_mail_content_type' );

        $hours = get_option( 'mdhqd_email_delay_hours', 0 );

        $users = get_users( array(
            'meta_query' => array(
                array(
                    'key' => 'mdhqd_last_update',
                    'value' => date( 'Y-m-d H:i:s', strtotime( "-{$hours} hours" ) ),
                    'compare' => '<',
                    'type' => 'DATETIME',
                ),
            ),
        ) );

        $cart_url = get_permalink( wc_get_page_id( 'cart' ) );
        $site_name = get_bloginfo( 'name' );
        $admin_email = get_option('admin_email');

        foreach ( $users as $user ) {
            $first_name = get_user_meta( $user->data->ID, 'first_name', true );
            $last_name = get_user_meta( $user->data->ID, 'last_name', true );
            $name = "{$first_name} {$last_name}";
            $nrlogin = get_user_meta( $user->data->ID, 'nr_link', true );
            $url_text = get_option( 'mdhqd_email_url_text', '' );
            $url = "<a href=\"{$cart_url}?nrlogin={$nrlogin}\">{$url_text}</a>";
            $memberships = get_posts( array(
                'post_type' => 'wc_user_membership',
                'post_status' => 'wcm-active',
                'author' => $user->data->ID,
            ) );
            $user_membership = array();
            foreach ( $memberships as $membership ) {
                $membership_info = get_post( $membership->post_parent );
                $user_membership[] = $membership_info->post_title;
            }
            $user_membership = implode( ", ", $user_membership );
            $user_cart = get_user_meta( $user->data->ID, '_woocommerce_persistent_cart_1', true );

            if (empty($user_cart['cart'])) {
                return;
            }

            $cart = "<table><tr><th style=\"padding:5px;background:#6691BF;color:#FFF\">Product</th><th style=\"padding:5px;background:#6691BF;color:#FFF\">Quantity</th>";
            foreach ( $user_cart['cart'] as $item_key => $item ) {
                $background = '#C6DBF1';
                $product = get_post( $item['product_id'] );
                $cart .= "<tr><td style=\"background:{$background};padding:5px;\">{$product->post_title}</td><td style=\"background:{$background};padding:5px;text-align:center;\">{$item['quantity']}</td>";
            }
            $cart .= "</table>";

            $subject = get_option( 'mdhqd_email_subject', '' );
            $subject = str_replace( '[name]', $name, $subject );
            $subject = str_replace( '[1click]', $url, $subject );
            $subject = str_replace( '[membership]', $user_membership, $subject );

            $message = get_option( 'mdhqd_email_message', '' );
            $message = str_replace( '[name]', $name, $message );
            $message = str_replace( '[1click]', $url, $message );
            $message = str_replace( '[membership]', $user_membership, $message );
            $message = str_replace( '[cart]', $cart, $message );

            wp_mail(
                $user->data->user_email,
                $subject,
                $message,
                'From: '. $site_name .', supplement fulfillment center <'. $admin_email .'>'
            );

            delete_user_meta( $user->data->ID, 'mdhqd_last_update' );
        }

        remove_filter( 'wp_mail_content_type', 'mdhqd_set_html_mail_content_type' );
    }
}

if ( ! function_exists( 'mdhqd_activation' ) ) {
    function mdhqd_activation() {
        if ( ! wp_next_scheduled( 'mdhqd_process_relay' ) ) {
            wp_schedule_event( time(), 'hourly', 'mdhqd_process_relay' );
        }
    }
}

function delay_cart_view()
{
    if (!empty($_POST['mdhqd_email_delay_hours'])) {
        $delayHours = $_POST['mdhqd_email_delay_hours'];
        $subject = $_POST['mdhqd_email_subject'];
        $message = stripslashes($_POST['mdhqd_email_message']);
        $clickText = $_POST['mdhqd_email_url_text'];
        update_option('mdhqd_email_delay_hours', $delayHours);
        update_option('mdhqd_email_subject', $subject);
        update_option('mdhqd_email_message', $message);
        update_option('mdhqd_email_url_text', $clickText);
    } else {
        $delayHours = get_option('mdhqd_email_delay_hours', '');
        $subject = get_option('mdhqd_email_subject', '');
        $message = get_option('mdhqd_email_message', '');
        $clickText = get_option('mdhqd_email_url_text', '');
    }

    wp_enqueue_style( 'orderemail_bootstrap', 'https://maxcdn.bootstrapcdn.com/bootstrap/4.5.2/css/bootstrap.min.css' );
    ?>
    <div class="container">
        <form action="" method="post">
            <div class="form-group">
                <label for="mdhqd_email_delay_hours">Delay Hours *</label>
                <input type="number" value="<?php echo (int) $delayHours; ?>" class="form-control" placeholder="" id="mdhqd_email_delay_hours" name="mdhqd_email_delay_hours" required>
            </div>
            <div class="form-group">
                <label for="mdhqd_email_subject">Subject *</label>
                <input type="text" value="<?php echo $subject; ?>" class="form-control" placeholder="" id="mdhqd_email_subject" name="mdhqd_email_subject" required>
            </div>
            <div class="form-group">
                <label for="mdhqd_email_message">Message *</label>
                <div>
                    You can use the following tags:<br><br>
                    <b>[name]</b> - Full name of the patient<br>
                    <b>[1click]</b> - 1click login link of the patient<br>
                    <b>[membership]</b> - User memberships<br>
                    <b>[cart]</b> - User cart table<br>
                </div>
                <?php wp_editor( $message , 'mdhqd_email_message', $settings = array('textarea_rows'=> '10') ); ?>
                <div class="form-group">
                    <label for="mdhqd_email_subject">1click Login Text *</label>
                    <input type="text" value="<?php echo $clickText; ?>"  class="form-control" placeholder="" id="mdhqd_email_url_text" name="mdhqd_email_url_text" required>
                </div>
            </div>

            <button type="submit" class="btn btn-primary">Submit</button>
        </form>
    </div>
<?php }


function new_contact_methods( $contactmethods ) {
    $contactmethods['mdhqd_email_notification'] = 'Email Notification';
    return $contactmethods;
}
add_filter( 'user_contactmethods', 'new_contact_methods', 10, 1 );


function new_modify_user_table( $column ) {
    $column['mdhqd_email_notification'] = 'Email Notification';
    return $column;
}
add_filter( 'manage_users_columns', 'new_modify_user_table', 9999 );

function new_modify_user_table_row( $val, $column_name, $user_id ) {
    switch ($column_name) {
        case 'mdhqd_email_notification' :
            return '<a class="button" href="' . site_url() . '/wp-admin/user-edit.php?user_id=' . $user_id . '&mdhqd_send_now=1">Send Email</a>';
            break;
        default:
    }
    return $val;
}
add_filter( 'manage_users_custom_column', 'new_modify_user_table_row', 999, 3 );
