<?php

/*
    Plugin Name: MDHQC Relay
    Plugin URI: https://fxmedsupport.com
    Description: Receive request from nutrimentRx. Added support for delay email and one click login
    Version: 3.0
    Author: Muhammad Yaseen
*/

defined( 'ABSPATH' ) || die('No direct access');

define('MDHQC_RELAY_SERVER_DIR',        __DIR__);

define( 'MDHQC_PLUGIN_PATH', plugin_dir_url( __FILE__ ) );

if(!class_exists('MDHQCRelayServer')) {

    if (!class_exists('MDHQCRelayFileLogger')) {
        include __DIR__ . '/MDHQCRelayFileLogger.php';
    }

    class MDHQCRelayServer extends MDHQCRelayFileLogger
    {
        private $products;

        public function __construct()
        {
            parent::__construct();
          
            spl_autoload_register(array($this, 'class_loader'));
            date_default_timezone_set(get_option('timezone_string'));
            add_action( 'manage_posts_extra_tablenav',  array($this, 'admin_product_list_top_bar_button'), 20, 1 );
            add_action( 'restrict_manage_posts', array($this, 'display_admin_product_language_filter') );
            if (!is_admin()) {
                add_action('init', array($this, 'handle_request'));
            }
            add_action('admin_menu', array($this, 'mdhqc_admin_settings'));
            add_action('init', array($this, 'save_admin_settings'));
            add_action('init', array($this, 'execute_cart_action'));
            add_action( 'rest_api_init', function () {
                register_rest_route( 'api/v1', '/cart-control-refill', array(
                    'methods' => 'POST',
                    'callback' => array($this, 'refill_cart'),
                    'permission_callback' => '__return_true',
                ) );
            } );
            $this->fileActions();
            $this->products = new MDHQCRelayProductSync();
            new MDHQCRelayRedbox();
            new MDHQCRelayNRSettings();
            new MDHQCRelayMembershipNotifications();
            new MDHQCRelaySyncNrx();

            // New Mem email
            if(in_array( 'woocommerce/woocommerce.php', apply_filters('active_plugins',get_option('active_plugins')))
            &&
                in_array( 'woocommerce-memberships/woocommerce-memberships.php', apply_filters('active_plugins',get_option('active_plugins'))))
            {
                require_once('includes/functions_mem_email.php');
//                $plugin_base_dir = plugin_dir_url(__FILE__);
//                $plugin_base_path = plugin_dir_path(__FILE__);
//                global $plugin_base_dir;
                add_action('admin_menu','assign_wc_membership_menu');
                add_shortcode('assign_wc_options_form','assign_wc_options_form');

                add_action( 'wp_ajax_assign_wc_options_register_user', 'assign_wc_options_register_user' );
                add_action( 'wp_ajax_nopriv_assign_wc_options_register_user', 'assign_wc_options_register_user' );
            }
        }

        public function refill_cart(WP_REST_Request $request)
        {
            $params = $request->get_json_params();
            
            if (empty($params['email']) || empty($params['products'])) {
                return ['success' => 0, 'message' => 'Invalid Params'];                    
            }

            $user = get_user_by( 'email', $params['email']);
            if (!$user) {
                return ['success' => 0, 'message' => 'User not found'];                    
            }

            $supplements = $params['products'];
            $products = [];
            foreach ($supplements as $supplement) {
                $product_id = wc_get_product_id_by_sku($supplement['code']);
                if ($product_id) {
                    $products[] = $product_id;
                }
            }

            if (empty($products)) {
                return ['success' => 0, 'message' => 'No products found.'];      
            }

            $products = implode(',', $products);
            $this->curl_wc_cart_action_get_url_output($user->data->ID, $products, 'add', true);
            return ['success' => 1];
        }

        private function fileActions() {
            if(is_admin() && isset($_GET['download_mdhqc_log'])) {
                echo $this->download();
                exit;
            } elseif(is_admin() && isset($_GET['download_mdhqc_log_found'])) {
                echo $this->download('logFound');
                exit;
            } elseif(is_admin() && isset($_GET['download_mdhqc_log_not_found'])) {
                echo $this->download('logNotFound');
                exit;
            } elseif(is_admin() && isset($_GET['download_product_log'])) {
                echo $this->download('product');
                exit;
            } elseif (is_admin() && isset($_GET['reset_mdhqc_log'])) {
                $this->resetLog();
                header('Location: ' . $_SERVER['HTTP_REFERER']);
                exit;
            } elseif (is_admin() && isset($_GET['reset_mdhqc_log_found'])) {
                $this->resetLog('logFound');
                header('Location: ' . $_SERVER['HTTP_REFERER']);
                exit;
            } elseif (is_admin() && isset($_GET['reset_mdhqc_log_not_found'])) {
                $this->resetLog('logNotFound');
                header('Location: ' . $_SERVER['HTTP_REFERER']);
                exit;
            } elseif (is_admin() && isset($_GET['reset_product_log'])) {
                $this->resetLog('product');
                header('Location: ' . $_SERVER['HTTP_REFERER']);
                exit;
            }
        }

        public function save_admin_settings() {
            if(isset($_GET['page']) &&
                $_GET['page'] === 'mdhqc-relay' && !empty($_POST['password'])) {
                $option = get_option('mdhqc_settings');
                $option = maybe_unserialize($option);

                if (!empty($_POST['plan'])) {
                    $option['plan'] = $_POST['plan'];
                } else {
                    $option['plan'] = '';
                }

                if (!empty($_POST['affiliate'])) {
                    $option['affiliate'] = $_POST['affiliate'];
                } else {
                    $option['affiliate'] = '';
                }

                $option['password'] = $_POST['password'];
                $output = $this->products->postCurl('https://nutrimentrx.com/?receive_mdhqc_input=1', json_encode(array_merge(array('site' => get_site_url()), $option)));
                $output = json_decode($output, true);
                if (!empty($output) && !empty($output['update']) && $output['update'] === '1') {
                    update_option('mdhqc_settings', $option);
                }
            }
        }

        public function mdhqc_admin_settings() {
            add_menu_page(
                __( 'CMDHQ Webhooks', 'textdomain' ),
                'MDHQC Relay',
                'manage_options',
                __DIR__,
                array($this, 'display_admin_page'),
                'dashicons-dashboard',
                6
            );

            add_submenu_page(
                'mdhqc-relay',
                'Delay Cart Email',
                'Delay Cart Email',
                'manage_options',
                'delay_cart_email',
                'delay_cart_view'
            );
        }

        public function display_admin_page() {
            $affiliates = null;
            $plans = null;
            $option = get_option('mdhqc_settings');

            if ($option === false) {
                $option = array('plan' => '', 'affiliate' => '', 'password' => '');
                update_option('mdhqc_settings', $option);
            } else {
                $option = maybe_unserialize($option);
            }

            if (function_exists('affiliate_wp')) {
                $affiliates = affiliate_wp()->affiliates->get_affiliates( ['number' => 0] );
            }

            if (function_exists('wc_memberships')) {
                $plans = wc_memberships_get_membership_plans();
            }

            ob_start();
            include (__DIR__ . '/views/admin.php');
            echo ob_get_clean();
        }

        private function class_loader($class) {
            if(strpos($class,'MDHQCRelay') === false)
                return;

            if(!class_exists($class))
                include $class . '.php';
        }

        public function admin_product_list_top_bar_button($which)
        {
            global $pagenow, $typenow;

            if ( 'product' === $typenow && 'edit.php' === $pagenow && 'top' === $which ) {
                ?>
                <div class="alignleft actions custom">
                    <button type="submit" name="mdhqc_push_all" style="height:32px;" class="button" value="mdhqc"><?php
                        echo __( 'Push All to MDHQ', 'woocommerce' ); ?></button>
                </div>
                <div class="alignleft actions custom">
                    <button type="submit" name="mdhqc_push_selected" style="height:32px;" class="button" value="mdhqc"><?php
                        echo __( 'Push Selected Products to MDHQ', 'woocommerce' ); ?></button>
                </div>
                <?php
            }
        }

        public function display_admin_product_language_filter() {
            global $pagenow, $typenow;

            if ( 'product' === $typenow && 'edit.php' === $pagenow) {
                if ((isset($_GET['mdhqc_push_all']) && $_GET['mdhqc_push_all'] === 'mdhqc')) {
                    $this->pushProducts();
                } elseif(isset($_GET['mdhqc_push_selected']) && $_GET['mdhqc_push_selected'] === 'mdhqc') {
                    if (isset($_GET['post']) && is_array($_GET['post'])) {
                        $this->pushProducts($_GET['post']);
                    } else {
//                        $this->addFlash('Please select some products!', 'error');
                    }
                }

            }
        }

        public function pushProductsBackground($products = null)
        {
            if (!empty($products)) {
                $file_name  = time() . '-products.txt';
                $file_path = __DIR__ . "/input-files/" . $file_name;
                file_put_contents($file_path, implode(',', $products));
            } else {
                $file_name  = time() . '.txt';
                $file_path = __DIR__ . "/input-files/" . $file_name;
                file_put_contents($file_path, 0);
            }

            chmod($file_path, 0666);
            $command = 'php ' . __DIR__ . '/cli.php --datafile=' . $file_name;
            $output_file = __DIR__ . '/output/' . time() . '-' . $file_name . '.log';
            file_put_contents(__DIR__ . "/command.log", $command . ' > ' . $output_file . ' 2>&1 & echo $!');
            $pid = (int)shell_exec($command . ' > ' . $output_file . ' 2>&1 & echo $!');
            // $pid = shell_exec("nohup ".$command." > /dev/null 2>&1 & echo $!");
            file_put_contents(__DIR__ . "/process.log", $pid);
        }

        public function pushProducts($products = null)
        {
            ini_set('max_execution_time', 0);
            global $wpdb;
            if (!empty($products)) {
                $products  = $wpdb->get_results(
                    "SELECT `a`.`ID`, `a`.`post_title`, `a`.`post_status`, `a`.`post_name`, `b`.`meta_value` AS `sku`, `c`.`meta_value` AS `stock`
                                    FROM `{$wpdb->posts}` AS `a` 
                                    LEFT JOIN `{$wpdb->postmeta}` AS `b` ON `a`.`ID` = `b`.`post_id` AND `b`.`meta_key` LIKE '_sku' 
                                    LEFT JOIN `{$wpdb->postmeta}` AS `c` ON `a`.`ID` = `c`.`post_id` AND `c`.`meta_key` LIKE '_stock_status' 
                                    WHERE `a`.`post_type` LIKE 'product' AND
                                          `a`.`ID` IN (". implode(',', $products) .")"
                );

                if (!empty($products)) {
                    $this->processProducts($products);
                }
                return;
            }

            $products  = $wpdb->get_results(
                "SELECT `a`.`ID`, `a`.`post_title`, `a`.`post_status`, `a`.`post_name`, `b`.`meta_value` AS `sku`, `c`.`meta_value` AS `stock`
                                    FROM `{$wpdb->posts}` AS `a` 
                                    LEFT JOIN `{$wpdb->postmeta}` AS `b` ON `a`.`ID` = `b`.`post_id` AND `b`.`meta_key` LIKE '_sku' 
                                    LEFT JOIN `{$wpdb->postmeta}` AS `c` ON `a`.`ID` = `c`.`post_id` AND `c`.`meta_key` LIKE '_stock_status'
                                    WHERE `a`.`post_type` LIKE 'product'"
            );

            if (!empty($products)) {
                $this->processProducts($products);
            }
        }

        private function processProducts($products)
        {
            foreach($products as $product) {
                if (empty($product->post_name) || empty($product->sku)) {
                    continue;
                }

                $taxonomy    = 'product_brand';
                $brand_names = wp_get_post_terms( $product->ID, $taxonomy, array( 'fields' => 'names' ) );
            
                // Get the brand name
                $brand_name = reset( $brand_names );
                $sku    = $product->sku;
                $stock  = $product->stock;
                $brand  = empty($brand_name) ? 'NO BRAND' : $brand_name;
                $name = $product->post_title . ' - ' . $sku . ' - ' . $brand;
                $slug = str_ireplace('__trashed','',$product->post_name);
                $nickname = str_ireplace('-',' ', $slug) . ', ' . $sku . ', ' . $brand;

                $inactive = true;
                if($product->post_status === 'publish' && $stock === 'instock') {
                    $inactive = false;
                }
                $data_string = array(
                    'name'          => $name, // update the title of the product
                    'nicknames'     => $nickname,
                    'vendor_code'   => $sku,
                    'description'   => '',
                    'class'         => '',
                    'external_ref_id' => $product->ID,
                    'inactive'      => $inactive,
                    'site' => get_site_url(),
                );

                $updated = $this->products->postCurl('https://nutrimentrx.com/?receive_product_mdhqc=1', json_encode($data_string));
                $updated = json_decode($updated);
                if(!empty($updated) && !isset($updated->error)) {
                    $this->log($sku . '-' . $name . ' product pushed' , 'product');
                } else {
                    $this->log($sku . '-' . $name . ' product not pushed' , 'product');
                }
                usleep(10000);
            }

            return true;
        }

        /**
         * Handle request coming from NutrimentRx and process it
         */
        public function handle_request()
        {
            if (isset($_GET['relay_mdhqc'])) {
                $inputJSON = file_get_contents('php://input');
                $data = json_decode($inputJSON);
                file_put_contents(__DIR__ . "/log/log.log", "sasadsad");
                if (empty($data)) {
                    die("Bad Request");
                }
                $json = $data->json;
                $args = $data->args;
                if (in_array($json->event_type, ['pt.created', 'pt.modified'])) {
                    $this->pt_modified($json, $args);
                } else if ($json->event_type == 'pt_plan_other.created') {
					$this->log("Inside plan created");
                    try {
                        $pt_obj = new stdClass();
                        $pt_obj->data = $json->data->patient;
                        try {
                            $this->pt_modified($pt_obj, $args);
                        } catch(\Exception $e) {
                            echo $e->getMessage();
                            die;
                        }
                        $this->log("Inside plan created11");
                        $this->log("before plan_created", 'logFound', 'info');
                        $this->plan_created($json);
                    } catch(\Exception $e) {
                        echo $e->getMessage();
                        die;
                    }
                } else if ($json->event_type == 'pt_plan_other.removed') {
                    $this->plan_removed($json);
                }
            }
        }

        private function pt_modified($patient, $args)
        {
            $user = get_user_by( 'email', $patient->data->email1 );
            if(!$user) {
                $user_data = array('user_pass' => $patient->data->last.$args->pwd, 'user_login' => $patient->data->email1, 'user_email' => $patient->data->email1);
                global $wp_roles;
                $available_roles = $wp_roles->roles;
                if (!empty($available_roles['customer'])) {
                    $user_data['role'] = 'customer';
                }
                $user_id = wp_insert_user($user_data);
                update_user_meta($user_id,'first_name', $patient->data->first);
                update_user_meta($user_id,'last_name', $patient->data->last);
                if (class_exists('WC_Customer')) {
                    $customer = new WC_Customer($user_id);
                    $address1 = empty($patient->data->address1) ? "" : $patient->data->address1;
                    $address2 = empty($patient->data->address2) ? "" : $patient->data->address2;
                    $city = empty($patient->data->city) ? "" : $patient->data->city;
                    $state = empty($patient->data->state) ? "" : $patient->data->state;
                    $country = "US";
                    $post_code = empty($patient->data->zip) ? "" : $patient->data->zip;
                    $phone = empty($patient->data->phone_mobile) ? (empty($patient->data->phone_home) ? "" : $patient->data->phone_home) : $patient->data->phone_mobile;
                    $customer->set_billing_address_1($address1);
                    $customer->set_billing_address_2($address2);
                    $customer->set_billing_city($city);
                    $customer->set_billing_state($state);
                    $customer->set_billing_country($country);
                    $customer->set_billing_postcode($post_code);
                    $customer->set_shipping_address_1($address1);
                    $customer->set_shipping_address_2($address2);
                    $customer->set_shipping_postcode($post_code);
                    $customer->set_shipping_state($state);
                    $customer->set_shipping_country($country);
                    $customer->set_shipping_city($city);
                    $customer->set_shipping_first_name($patient->data->first);
                    $customer->set_shipping_last_name($patient->data->last);
                    $customer->set_billing_phone($phone);
                    // $customer->save();
                }
                $this->log("New WPUser#$user_id created with password=".$patient->data->last.$args->pwd);
            } else {
                $user_id = $user->data->ID;
                $this->log("WPUser#$user_id found");
            }
            // echo $patient->data->first;die;
            $this->log("WPUser#$user_id hering");
            update_user_meta($user_id,'first_name', $patient->data->first);
            update_user_meta($user_id,'last_name', $patient->data->last);
            // #1 error cause
            // wp_update_user([
            //     'ID' => $user_id,
            //     'first_name' => $patient->data->first,
            //     'last_name' => $patient->data->last
            // ]);
			$this->log("WPUser#$user_id here");
            $mdhq_ids = get_user_meta($user_id,'mdhq_user_id');
            if(!in_array($patient->data->id, $mdhq_ids)) {
                add_user_meta($user_id,'mdhq_user_id', $patient->data->id);
            }
			$this->log("WPUser#$user_id here1");
            if (!empty(intval($args->enable_affiliates))) {
                $user_affiliate_id = get_user_meta($user_id,'affwp_lc_affiliate_id',true);
                if(!$user_affiliate_id) {
                    add_user_meta($user_id,'affwp_lc_affiliate_id', $args->src,true);
                    $this->log("Affilate#" . $args->src . " added for this user.");
                } else {
                    if($args->src != $user_affiliate_id) {
                        update_user_meta($user_id,'affwp_lc_affiliate_id', $args->src, $user_affiliate_id);
                        $this->log("Affilate#" . $args->src . " updated for this user.");
                    }
                }
            }

            if (!function_exists('wc_memberships') || empty(intval($args->enable_membership)))
                return;
$this->log("WPUser#$user_id here2");
            $plan = wc_memberships_get_membership_plan($args->plan);
            if($plan) {
                $this->log("WPUser#$user_id here3");
                $user_membership = wc_memberships_get_user_membership( $user_id, $plan->id );
                if(!$user_membership) {
                    $plan_args = array(
                        'plan_id' => $plan->id,
                        'user_id' => $user_id,
                    );
                    $membership_wc = wc_memberships_create_user_membership($plan_args);
                    $new_date = date('Y-m-d H:i:s',(strtotime('-1 day', strtotime ($membership_wc->get_start_date()))));
                    $membership_wc->set_start_date($new_date);
                    $membership_wc->update_status('active');
                    $this->log("Plan#$plan->id added for WPUser#$user_id");
                } else {
                    $this->log("WPUser#$user_id is already added to this plan");
                }
            }
        }

        private function plan_created($supplement) {
            $user = get_user_by( 'email', $supplement->data->patient->email1 );
            $user_id = $user->data->ID;
            $product_code = $supplement->data->plan_definition->code;
            if($product_code) {
                $product_id = wc_get_product_id_by_sku( $product_code );
                if($product_id) {
                    $this->cart_action($user_id, $product_id);
                    $this->log("Product($product_code) added to WPUser#$user_id cart");
                } else {
                    // Practice ID, user email, product sku and product name
                    $this->log('[ ' . $supplement->practice_id . ' - ' .
                        $supplement->data->patient->email1 . ' - ' .
                        $supplement->data->plan_definition->code . ' - ' .
                        $supplement->data->plan_definition->name . ' ]', 'logNotFound');
                    $this->log("Product($product_code) not found.");
                    $this->log("Product($product_code) not found.", 'logNotFound');
                }
            } else {
                $this->log("Empty product code");
            }
        }

        private function plan_removed($supplement) {
            $user = get_user_by( 'email', $supplement->data->patient->email1 );
            if($user) {
                if($supplement->data->plan_definition->code) {
                    $product_id = wc_get_product_id_by_sku( $supplement->data->plan_definition->code );
                    if($product_id) {
                        $this->cart_action($user->data->ID, $product_id, 'remove');
                        $this->log("Product(".$supplement->data->plan_definition->code.") removed from WPUser#".$user->data->ID." cart");
                    } else {
                        //$this->log("User Email=".$supplement->data->patient->email1, 'logNotFound');
                        $this->log('[ ' . $supplement->practice_id . ' - ' .
                            $supplement->data->patient->email1 . ' - ' .
                            $supplement->data->plan_definition->code . ' - ' .
                            $supplement->data->plan_definition->name . ' ]', 'logNotFound');
                        $product_code = $supplement->data->plan_definition->code;
                        $this->log("Product($product_code) not found.");
                        $this->log("Product($product_code) not found.", 'logNotFound');
                    }
                } else {
                    $this->log("Empty product code");
                }
            }
        }

        private function curl_wc_cart_action_get_url_output($user_id, $product_id, $action = 'add', $clear = false) {
            $nr_link = get_user_meta( $user_id, 'nr_link', true);
            if($nr_link != ''){
                $loginUrl = get_site_url()."/my-account/?nrlogin=".$nr_link;
            }

            //init curl
            $ch = curl_init();
            //Set the URL to work with
            curl_setopt($ch, CURLOPT_URL, $loginUrl);
//             curl_setopt($ch, CURLOPT_POST, 1);
            //curl_setopt($ch, CURLOPT_POSTFIELDS, 'user='.$username.'&pass='.$password);
            file_put_contents(__DIR__.'/cookies/'.$user_id.'.txt', '');
            curl_setopt($ch, CURLOPT_COOKIEJAR, __DIR__.'/cookies/'.$user_id.'.txt');
			curl_setopt($ch, CURLOPT_COOKIEFILE, __DIR__.'/cookies/'.$user_id.'.txt');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $store = curl_exec($ch);
            if($action == 'add') {
                $clearStr = '';
                if ($clear) {
                    $clearStr = '&clear=1';
                }
                curl_setopt($ch, CURLOPT_URL, get_site_url()."/cart/?curl_mdhq_wc_add_to_cart=".$product_id . $clearStr);
            } else {
                curl_setopt($ch, CURLOPT_URL, get_site_url()."/cart/?curl_mdhq_wc_remove_from_cart=".$product_id);
            }
            $content = curl_exec($ch);
            file_put_contents(__DIR__.'/cart.html', $content);
        }

        public function execute_cart_action() {
            $shouldClear = !empty($_GET['clear']);
            $wc = WC();
            if ($shouldClear) {
                if (sizeof($wc->cart->get_cart()) > 0) {
                    foreach ($wc->cart->get_cart() as $cart_item_key => $values) {
                        $wc->cart->remove_cart_item($cart_item_key);
                    }
                }

                $product_ids = explode(',', $_GET['curl_mdhq_wc_add_to_cart']);
                if (!empty($product_ids)) {
                    foreach ($product_ids as $product_id) {
                        $wc->cart->add_to_cart($product_id);
                    }
                }
                return;
            }

            $product_id = !empty($_GET['curl_mdhq_wc_add_to_cart']) ?
                filter_input(INPUT_GET, 'curl_mdhq_wc_add_to_cart', FILTER_SANITIZE_NUMBER_INT) :
                false;

            if(!$product_id)
                $product_id = !empty($_GET['curl_mdhq_wc_remove_from_cart']) ?
                    filter_input(INPUT_GET, 'curl_mdhq_wc_remove_from_cart', FILTER_SANITIZE_NUMBER_INT) :
                    false;

            if(!$product_id)
                return;

            $shouldAdd = isset($_GET['curl_mdhq_wc_add_to_cart']);
            $found = false;
            if (sizeof($wc->cart->get_cart()) > 0) {
                foreach ($wc->cart->get_cart() as $cart_item_key => $values) {
                    $_product = $values['data'];
                    if ((int)$_product->id === (int)$product_id) {
                        $found = true;
                        break;
                    }
                }
                // if product not found, add it
                if ($found) {
                    $quantity = (int)$values['quantity'];
                    if(!$shouldAdd && (int)$quantity === 1) {
                        $wc->cart->remove_cart_item($cart_item_key);
                    } else {
                        $newQty = $shouldAdd ? ++$quantity : --$quantity;
                        $wc->cart->set_quantity($cart_item_key, $newQty);
                    }
                } else {
                    $shouldAdd ? $wc->cart->add_to_cart($product_id) : false;
                }
            } else {
                $shouldAdd ? $wc->cart->add_to_cart($product_id) : false;
            }
        }

        /**
         * Add products to given user's cart
         *
         * @param $user_id
         * @param $product_id
         * @param string $action
         * @throws Exception
         */
        public function cart_action($user_id, $product_id, $action = 'add')
        {
            $curr_user = wp_set_current_user( $user_id, '' );
            wp_set_auth_cookie( $user_id);

            $wc = WC();
            $shouldAdd = $action == "add" ? true : false;
            $found = false;
            if (sizeof($wc->cart->get_cart()) > 0) {
                foreach ($wc->cart->get_cart() as $cart_item_key => $values) {
                    $_product = $values['data'];
                    if ((int)$_product->id === (int)$product_id) {
                        $found = true;
                        break;
                    }
                }
                // if product not found, add it
                if ($found) {
                    $quantity = (int)$values['quantity'];
                    if(!$shouldAdd && (int)$quantity === 1) {
                        $wc->cart->remove_cart_item($cart_item_key);
                    } else {
                        $newQty = $shouldAdd ? ++$quantity : --$quantity;
                        $wc->cart->set_quantity($cart_item_key, $newQty);
                    }
                } else {
                    $shouldAdd ? $wc->cart->add_to_cart($product_id) : false;
                }
            } else {
                $shouldAdd ? $wc->cart->add_to_cart($product_id) : false;
            }
			$nr_link = get_user_meta( $user_id, 'nr_link', true);
            if($nr_link != ''){
                $loginUrl = get_site_url()."/my-account/?nrlogin=".$nr_link;
                file_get_contents($loginUrl);
            }
            $this->updateMetaDelay($user_id);
        }

        private function updateMetaDelay($user_id)
        {
            update_user_meta(
                $user_id,
                'mdhqd_last_update',
                date( 'Y-m-d H:i:s' )
            );
        }
    }
}

new MDHQCRelayServer();
